<?php
#BEGIN_LICENSE
#-------------------------------------------------------------------------
# Module: FrontEndUsers (c) 2008-2016 by Robert Campbell
#         (calguy1000@cmsmadesimple.org)
#  An addon module for CMS Made Simple to allow management of frontend
#  users, and their login process within a CMS Made Simple powered
#  website.
#
#-------------------------------------------------------------------------
# CMS - CMS Made Simple is (c) 2005 by Ted Kulp (wishy@cmsmadesimple.org)
# Visit the CMSMS Homepage at: http://www.cmsmadesimple.org
#
#-------------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# However, as a special exception to the GPL, this software is distributed
# as an addon module to CMS Made Simple.  You may not use this software
# in any Non GPL version of CMS Made simple, or in any version of CMS
# Made simple that does not indicate clearly and obviously in its admin
# section that the site was built with CMS Made simple.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
# Or read it online: http://www.gnu.org/licenses/licenses.html#GPL
#
#-------------------------------------------------------------------------
#END_LICENSE
use \FrontEndUsers\verification_assistant;
use \FrontEndUsers\email_verify_assistant;
use \CMSMS\HookManager;

/**
 * This file defines the FrontEndUsers module class.
 *
 * @package FrontEndUsers
 */

/**
 * The FrontEndUsers module, besides being a module to provide authentication services to the frontend of a CMSMS website also has a complete
 * and flexible API.
 *
 * @package FrontEndUsers
 */
final class FrontEndUsers extends CGExtensions
{
    /**
     * Constant to indentify a text property.
     */
    const FIELDTYPE_TEXT = 0;

    /**
     * Constant to indentify a checkbox property.
     */
    const FIELDTYPE_CHECKBOX = 1;

    /**
     * Constant to indentify an email property.
     */
    const FIELDTYPE_EMAIL = 2;

    /**
     * Constant to indentify a textarea property.
     */
    const FIELDTYPE_TEXTAREA = 3;

    /**
     * Constant to indentify a dropdown property.
     */
    const FIELDTYPE_DROPDOWN = 4;

    /**
     * Constant to indentify a multiselect property.
     */
    const FIELDTYPE_MULTISELECT = 5;

    /**
     * Constant to indentify an image property.
     */
    const FIELDTYPE_IMAGE = 6;

    /**
     * Constant to indentify a radio button group property.
     */
    const FIELDTYPE_RADIOBUTNS = 7;

    /**
     * Constant to indentify a date property.
     */
    const FIELDTYPE_DATE = 8;

    /**
     * Constant to indentify a data property.
     */
    const FIELDTYPE_DATA = 9;

    /**
     * How many seconds of inactivity before a user is automatically logged out.
     * @ignore
     */
    protected $expirytime;

    /**
     * Set this to true if user should be redirected to default page after logging in/out.
     * @ignore
     */
    protected $logintarget;

    /**
     * @ignore
     */
    protected $otherintarget;

    /**
     * @ignore
     */
    protected $logouttarget;

    /**
     * @ignore
     */
    protected $otherouttarget;

    /**
     * @ignore
     */
    protected $allowuseradmin;

    /**
     * @ignore
     */
    protected $enableemailpw;

    /**
     * @ignore
     */
    protected $lang="";

    /**
     * @ignore
     */
    private $usermanip = false;

    /**
     * @ignore
     */
    private $verification_assistant = null;

    /**
     * @ignore - work around constructor called twice issue
     */
    private static $_initialized;

    /**
     * Constructor
     *
     * @ignore
     */
    public function __construct()
    {
        parent::__construct();

        if( self::$_initialized ) return;
        self::$_initialized = true;
    }

    /**
     * @ignore
     */
    private function _load()
    {
        if( $this->usermanip == NULL ) {
            // check a preference to see which user manipulator class
            // we should use.
            $this->usermanip = new FrontEndUsersManipulator( $this );
            if( !is_object( $this->usermanip ) ) die("Error instantiationg manipulator object");
            $this->usermanip->SetExpireNotifier(array($this,'NotifyExpiredUser'));
        }
    }

    protected function GetFieldTypes()
    {
        return [ 'text' => self::FIELDTYPE_TEXT,
                 'checkbox' => self::FIELDTYPE_CHECKBOX,
                 'email' => self::FIELDTYPE_EMAIL,
                 'textarea' => self::FIELDTYPE_TEXTAREA,
                 'dropdown' => self::FIELDTYPE_DROPDOWN,
                 'multiselect' => self::FIELDTYPE_MULTISELECT,
                 'image' => self::FIELDTYPE_IMAGE,
                 'radiobuttons' => self::FIELDTYPE_RADIOBUTNS,
                 'date' => self::FIELDTYPE_DATE,
                 'data' => self::FIELDTYPE_DATA
            ];
    }

    /**
     * Get the User Manipulator Object
     * NOT FOR EXTERNAL USE
     *
     * @internal
     * @return object
     */
    public function GetManipulator()
    {
        $this->_load();
        return $this->usermanip;
    }

    /**
     * @ignore
     */
    function GetName() { return 'FrontEndUsers'; }

    /**
     * @ignore
     */
    function GetVersion() { return '2.12.7'; }

    /**
     * @ignore
     */
    function HasContentType() { return TRUE; }

    /**
     * @ignore
     */
    function IsPluginModule() { return TRUE; }

    /**
     * @ignore
     */
    function AllowAutoInstall() { return FALSE; }

    /**
     * @ignore
     */
    function AllowAutoUpgrade() { return FALSE; }

    /**
     * @ignore
     */
    function LazyLoadAdmin() { return FALSE; }

    /**
     * @ignore
     */
    function MinimumCMSVersion() { return '2.2.7'; }

    /**
     * @ignore
     */
    function GetAdminDescription () { return $this->Lang ('moddescription'); }

    /**
     * @ignore
     */
    function GetAdminSection () { return 'usersgroups'; }

    /**
     * @ignore
     */
    function GetDependencies() { return ['CGExtensions' => '1.60','CGSimpleSmarty' => '2.1.6' ]; }

    /**
     * @ignore
     */
    function GetEventDescription ( $eventname ) { return $this->Lang('event_info_'.$eventname ); }

    /**
     * @ignore
     */
    function GetEventHelp ( $eventname ) { return $this->Lang('event_help_'.$eventname ); }

    /**
     * @ignore
     */
    function GetFriendlyName () { return $this->Lang('friendlyname'); }

    /**
     * @ignore
     */
    function HasAdmin () { return TRUE; }

    /**
     * @ignore
     */
    function InstallPostMessage() { return $this->Lang('postinstallmessage'); }

    /**
     * @ignore
     */
    function GetAuthor() { return 'calguy1000'; }

    /**
     * @ignore
     */
    function GetAuthorEmail() { return 'calguy1000@hotmail.com'; }

    /**
     * @ignore
     */
    function GetChangeLog() { return file_get_contents(__DIR__.'/changelog.inc'); }

    /**
     * @ignore
     * @deprecated
     */
    protected function langifyKeys( $arr )
    {
        $out = array();
        foreach( $arr as $k=>$v ) {
            $k = $this->Lang($k);
            $out[ $k ] = $v;
        }
        return $out;
    }

    function SetParameters()
    {
        $smarty = cmsms()->GetSmarty();
        if( !$smarty ) return;

        $smarty->registerClass('feu_smarty','feu_smarty');
        $smarty->registerPlugin('block','feu_protect', '\\feu_smarty_plugins::feu_protect' );
        $this->AddImageDir('icons');

        $contentops = ContentOperations::get_instance();

        $obj = new CmsContentTypePlaceholder();
        $obj->class = 'FrontEndUsers\ProtectedPage';
        $obj->type  = 'feu_protected_page';  // must be lowercase, and short, this is what is stored in the database.
        $obj->filename = __DIR__.'/lib/class.feu_protected_page.php';
        $obj->loaded = false;
        $obj->friendlyname = $this->Lang('feu_protected_page');
        $contentops->register_content_type($obj);

        $obj = new CmsContentTypePlaceholder();
        $obj->class = 'FrontEndUsers\ProtectedSectionHead';
        $obj->type  = 'feu_protected_sectionhead';
        $obj->filename = __DIR__.'/lib/class.feu_protected_sectionhead.php';
        $obj->loaded = false;
        $obj->friendlyname = $this->Lang('feu_protected_sectionhead');
        $contentops->register_content_type($obj);

        $obj = new CmsContentTypePlaceholder();
        $obj->class = 'feu_protected_logoutlink';
        $obj->type  = strtolower($obj->class);
        $obj->filename = __DIR__.'/lib/class.feu_protected_logoutlink.php';
        $obj->loaded = false;
        $obj->friendlyname = $this->Lang('feu_protected_logoutlink');
        $contentops->register_content_type($obj);

        HookManager::add_hook('FrontEndUsers::OnDeleteUser', 'feu_utils::hook_ondeleteuser' );
        HookManager::add_hook('FrontEndUsers::AfterDeleteUser', 'feu_utils::hook_afterdeleteuser' );
    }

    /**
     * @ignore
     */
    function InitializeFrontend()
    {
        parent::InitializeFrontend();
        $this->RestrictUnknownParams();
        $this->RegisterModulePlugin();
        $this->SetParameterType('code',CLEAN_STRING);
        $this->SetParameterType('form',CLEAN_STRING); // deprecated
        $this->SetParameterType('returnto',CLEAN_STRING);
        $this->SetParameterType('only_groups',CLEAN_STRING);
        $this->SetParameterType('nocaptcha',CLEAN_INT);
        $this->SetParameterType('input_username',CLEAN_STRING); // deprecated
        $this->SetParameterType('input_password',CLEAN_STRING); // deprecated
        $this->SetParameterType('input_repeatpassword',CLEAN_STRING); // deprecated
        $this->SetParameterType('error',CLEAN_INT); // deprecated
        $this->SetParameterType('message',CLEAN_STRING); // deprecated
        $this->SetParameterType('lostun_group',CLEAN_STRING);
        $this->SetParameterType('input_captcha',CLEAN_STRING); // deprecated
        $this->SetParameterType('submit',CLEAN_STRING); // deprecated
        $this->SetParameterType('cancel',CLEAN_STRING); // deprecated
        $this->SetParameterType('input_code',CLEAN_STRING); // deprecated
        $this->SetParameterType('uid',CLEAN_INT);
        $this->SetParameterType('checkonly',CLEAN_INT);
        $this->SetParameterType('inline',CLEAN_INT);
        $this->SetParameterType('logouttemplate',CLEAN_STRING);
        $this->SetParameterType('logintemplate',CLEAN_STRING);
        $this->SetParameterType('changesettingstemplate',CLEAN_STRING);
        $this->SetParameterType('forgotpwtemplate',CLEAN_STRING);
        $this->SetParameterType('forgotpwemailtemplate',CLEAN_STRING);
        $this->SetParameterType('lostuntemplate',CLEAN_STRING);
        $this->SetParameterType('lostunconfirmtemplate',CLEAN_STRING);
        $this->SetParameterType('forcenewpwtemplate',CLEAN_STRING);
        $this->SetParameterType('viewusertemplate',CLEAN_STRING);
        $this->SetParameterType('afterverifytemplate',CLEAN_STRING);
        $this->SetParameterType('resetsessiontemplate',CLEAN_STRING);
        $this->SetParameterType('verifyformtemplate',CLEAN_STRING);
        $this->SetParameterType(CLEAN_REGEXP.'/feu_.*/',CLEAN_STRING);

        $this->RegisterRoute('/[fF]eu\/verifycode\/(?P<returnid>[0-9]+)\/(?P<uid>[0-9]+)\/(?P<code>.*?)$/',array('action'=>'verifycode'));
        $this->RegisterRoute('/[fF]eu\/verify\/(?P<returnid>[0-9]+)\/(?P<uid>[0-9]+)\/(?P<code>.*?)$/',array('action'=>'verifyonly'));
        $this->RegisterRoute('/[fF]eu\/reset\/(?P<uid>[0-9]+)$/',array('action'=>'reset_session','showtemplate'=>'false','checkonly'=>'1'));
        $this->RegisterRoute('/[fF]eu\/edit\/(?P<returnid>[0-9]+)$/',array('action'=>'changesettings'));
        $this->RegisterRoute('/[fF]eu\/logout\/(?P<returnid>[0-9]+)$/',array('action'=>'logout'));
        $this->RegisterRoute('/[fF]eu\/forgot\/(?P<returnid>[0-9]+)$/',array('action'=>'forgotpw'));
        $this->RegisterRoute('/[fF]eu\/lostusername\/(?P<returnid>[0-9]+)$/',array('action'=>'lostusername'));
    }

    /**
     * @ignore
     */
    function get_tasks()
    {
        $out = null;

        // clear history task
        $tmp = (int) $this->GetPreference('clearhistory_age');
        if( $tmp > 0 ) {
            $obj = new \FrontEndUsers\ClearUserHistoryTask();
            $out[] = $obj;
        }

        // force logout at specific server times
        $tmp = $this->GetPreference('forcelogout_times');
        if( $tmp != '' ) {
            $obj = new FEUForcedLogoutTask($this);
            $out[] = $obj;
        }

        return $out;
    }

    /**
     * @ignore
     */
	public function get_pretty_url($id,$action,$returnid='',$params=array(),$inline=false)
    {
        if( $action == 'default' ) {
            $form = \cge_param::get_string($params,'form');
            switch( $form ) {
            case 'forgotpw':
                $action = 'forgotpw';
                break;

            case 'login':
                $action = 'login';
                break;

            case 'lostusername':
                $action = 'lostusername';
                break;

            case 'changesettings':
                return "feu/edit/$returnid";
            }
        }

        if( $action == 'verifyonly' ) {
            $uid = \cge_param::get_int($params,'uid');
            $code = \cge_param::get_string($params,'code');
            if( $uid < 1 || !$code ) return;
            return "feu/verify/$returnid/$uid/$code";
        }
        if( $action == 'forgotpw') {
            return "feu/forgot/$returnid";
        } else if( $action == 'lostusername' ) {
            return "feu/lostusername/$returnid";
        } elseif( $action == 'logout' ) {
            $returnto = \cge_param::get_string($params,'returnto');
            if( $returnto ) return;
            return "feu/logout/$returnid";
        } elseif( $action == 'changesettings' ) {
            return "feu/edit/$returnid";
        }

    }

    /**
     * @ignore
     */
    protected function _HasSufficientPermissions( $perm = '' )
    {
        if ($this->GetPreference('feusers_specific_permissions','0') == '1') {
            $pusers = $this->CheckPermission( 'FEU Modify Users' );
            $pgroups = $this->CheckPermission( 'FEU Modify Groups' );
            $pprop = $this->CheckPermission( 'FEU Modify FrontEndUserProps' );
        }
        else {
            $pusers = $this->CheckPermission( 'Manage Users' );
            $pgroups = $this->CheckPermission( 'Manage Groups' );
            $pprop = $this->CheckPermission( 'Modify FrontEndUserProps' );
        }

        $consumer = feu_utils::get_auth_consumer();
        $ret = 0;
        switch( $perm ) {
        case '':  // any permission for working with groups and users
            $ret = ($pusers || $pgroups || $pprop);
            break;

        case 'listusers':
        case 'users':
        case 'adduser':
        case 'editusers':
        case 'removeusers':
            $ret = $pusers;
            break;

        case 'properties':
            $ret = $pprop && $consumer->has_capability($consumer::CAPABILITY_LISTPROPS);
            break;

        case 'addprop':
        case 'editprop':
        case 'deleteprop':
            $ret = $pprop && $consumer->has_capability($consumer::CAPABILITY_EDITPROPS);
            break;

        case 'listgroups':
            $ret = $pgroups &&
                $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS) || $consumer->has_capability($consumer::CAPABILITY_LISTGROUPS);
            break;

        case 'groups':
            $ret = $pgroups &&
                $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS) || $consumer->has_capability($consumer::CAPABILITY_LISTGROUPS);
            break;

        case 'addgroup':
            $ret = $pgroups && $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS) || $consumer->has_capability($consumer::CAPABILITY_EDITGROUPS);
            break;

        case 'strict_editgroups':
            $ret = $pgroups && $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS) || $consumer->has_capability($consumer::CAPABILITY_EDITGROUPS);
            break;

        case 'editgroups':
            $ret = $pgroups && $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS,$consumer::CAPABILITY_EDITGROUPS,$consumer::CAPABILITY_EDITGROUPPROPS);
            break;

        case 'removegroups':
            $ret = $pgroups && $consumer->has_capability($consumer::CAPABILITY_USESTDGROUPS,$consumer::CAPABILITY_EDITGROUPS);
            break;

        case 'usersngroups':
            $ret = ($pusers || $pgroups);
            break;

        default:
            $ret = 0;
            break;
        }

        return $ret;
    }

    /**
     * @ignore
     */
    function VisibleToAdminUser()
    {
        return $this->_HasSufficientPermissions('');
    }

    public function GetAdminMenuItems()
    {
        $out = array();
        if( $this->_HasSufficientPermissions('') ) $out[] = \CmsAdminMenuItem::from_module($this);
        if( $this->CheckPermission('Modify Site Preferences') ) {
            $obj = new CmsAdminMenuItem();
            $obj->module = $this->GetName();
            $obj->section = 'siteadmin';
            $obj->title = $this->Lang('title_adminnav_settings');
            $obj->description = $this->Lang('desc_adminnav_settings');
            $obj->action = 'admin_settings';
            $obj->url = $this->create_url('m1_',$obj->action);
            $out[] = $obj;
        }
        return $out;
    }

    /**
     * @ignore
     */
    protected function _DisplayErrorPage($id, &$params, $returnid, $message='')
    {
        echo $this->DisplayErrorMessage($message);
    }

    /**
     * @ignore
     */
    function DoAction($action, $id, $params, $returnid = -1)
    {
        if( isset($params['feu_enc']) ) {
            // restore parameters saved in the session on redirect.
            $sig = $params['feu_enc'];
            unset($params['feu_enc']);
            if( isset($_SESSION[$sig]) ) {
                $tmp = $_SESSION[$sig];
                unset($_SESSION[$sig]);
                $params = array_merge($tmp,$params);
            }
        }

        if( $action == 'default' ) {
            $form = 'login';
            if( $this->LoggedInId() ) $form = 'logout';
            $form = \cge_param::get_string($params,'form',$form);
            switch( $form ) {
            case 'login':
            case 'forgotpw':
            case 'silent':
            case 'changesettings':
            case 'lostusername':
                $action = $form;
                break;

            case 'logout':
                $action = 'logoutform';
                break;

            default:
                throw new \LogicExcception("$form is an invalid value for the form parameter");
            }
        }

        switch ($action) {
        case 'changesettings_url':
            break;

        default:
            parent::DoAction($action,$id,$params,$returnid);
            break;
        }
    }

    /**
     * @internal
     * @deprecated
     */
    function myRedirect( $id, $action, $returnid, $params = array(), $ignore_returnto = false )
    {
        // find any parameter values that are arrays
        // automatically store this data in the session
        unset($params['action'],$params['returnid']);

        $sig = md5(__METHOD__.time().serialize($params));
        $_SESSION[$sig] = $params;
        $params = [ 'feu_enc' => $sig ];
        $this->Redirect( $id, $action, $returnid, $params );
    }

    /**
     * @internal
     */
    public function GetHeaderHTML()
    {
        $txt = parent::GetHeaderHTML();
        $tmpl =<<<EOT
{JQueryTools action='require' lib='tablesorter,JQueryTools'}
{JQueryTools action='placemarker'}
EOT;

        $obj = cms_utils::get_module('JQueryTools','1.2');
        if( is_object($obj) ) {
            $smarty = Smarty_CMS::get_instance();
            $txt .= $smarty->fetch('string:'.$tmpl);
        }
        return $txt;
    }

    /**
     * send an event that this user account has been expired
     *
     * @internal
     */
    public function NotifyExpiredUser( $userid )
    {
        $user = $this->GetUserInfo( $userid );
        if( $user[0] == FALSE ) return; // this should be an error
        $parms = array();
        $parms['id'] = $userid;
        $parms['username'] = $user[1]['username'];
        HookManager::do_hook('FrontEndUsers::OnExpireUser',$parms);
    }

    /**
     * @internal
     */
    public function is_allowed_upload($srcfile)
    {
        $allowed_extensions=$this->GetPreference('allowed_image_extensions','.gif,.png,.jpg');
        $tmp = explode( ',', strtolower($allowed_extensions));
        $srcfile = strtolower($srcfile);

        foreach( $tmp as $ext ) {
            if( endswith( $srcfile, $ext ) ) return TRUE;
        }
        return FALSE;
    }

    /**
     * @internal
     * @todo: move me out of here
     */
    public function get_upload_dirurl()
    {
        $config = \cms_config::get_instance();
        $dn = cms_join_path($config['uploads_url'],$this->GetPreference('image_destination_path', 'feusers'));
        $dn = str_replace('\\','/',$dn);
        return $dn;
    }

    /**
     * @internal
     */
    public function get_upload_dirname()
    {
        $config = \cms_config::get_instance();
        $dn = cms_join_path($config['uploads_path'],$this->GetPreference('image_destination_path', 'feusers'));
        return $dn;
    }

    /**
     * @internal
     */
    public function get_upload_filename($fldname,$srcfile)
    {
        // get the filename only, not the directory
        // note: this must be reproducable given the same srcfile and filename for the same request.
        $ext = strrchr($srcfile,'.');
        $time = \cge_param::get_int( $_SERVER, 'REQUEST_TIME' );
        return sha1(__FILE__.session_id().$fldname.$srcfile.$time).$ext;
    }

    /**
     * @internal
     */
    public function ManageImageUpload($key, $fldname)
    {
        $res = \feu_utils::checkUpload($key);
        if( !$res[0] ) return $res;

        $file = $_FILES[$key];
        // set the destination name
        $destDir = $this->get_upload_dirname();
        $destname = $this->get_upload_filename($fldname,$file['name']);

        // Create the destination directory if necessary
        @mkdir($destDir,0777,TRUE);
        @touch($destdir.'/index.html');
        if( !is_writable( $destDir ) ) return [ false, $mod->Lang('error_destinationnotwritable') ];
        @cms_move_uploaded_file($file['tmp_name'], cms_join_path($destDir,$destname));

        return [ true, $destname ];
    }

    /**
     * @internal
     */
    public function SetPostLoginURL($url)
    {
        if( $url ) $this->session_put('postlogin_url',$url);
    }

    /**
     * @internal
     */
    public function GetPostLoginURL()
    {
        $out = $this->session_get('postlogin_url');
        $this->session_clear('postlogin_url');
        return $out;
    }

    //////////////////////////////////////////
    //  API FUNCTIONS //
    //////////////////////////////////////////

    /**
     * @ignore
     */
    public function __call( $name, $args )
    {
        $this->_load();
        //if( !method_exists( $this->usermanip, $name ) ) die( "method $name not found in ".get_class($this->usermanip)." object" );

        return call_user_func_array( [ $this->usermanip, $name ], $args );
    }

    /**
     * Get the filenames of all files associated with image properties for the specified user.
     * Does not check if the files exist, have thumbnails, or for permissions.
     *
     * @param int $uid The userid
     * @return array|null
     */
    function GetUserFilesFull( $userid )
    {
        $userid = (int) $userid;
        if( $userid < 1 ) return;

        $defns = $this->GetPropertyDefns();
        $propvals = $this->GetUserProperties( $userid );
        if( !$propvals ) return;
        $propvals = \cge_array::to_hash( $propvals, 'title' );
        $out = null;
        foreach( $defns as $name => $defn ) {
            if( $defn['type'] != self::FIELDTYPE_IMAGE ) continue;
            if( !isset( $propvals[$name]) ) continue;
            if( !$propvals[$name]['data'] ) continue;

            $out[$name] = $propvals[$name]['data'];
        }
        return $out;
    }


    /**
     * @internal
     * @ignore
     */
    function HasCapability($capability,$params = array())
    {
        switch( $capability ) {
        case 'content_types':
        case 'content_attributes':
        case 'tasks':
            return TRUE;
        default:
            return FALSE;
        }
    }

    /**
     * @internal
     * @ignore
     */
    function get_content_attributes($content_type)
    {
        $tmp = array();
        $attr = new CmsContentTypeProfileAttribute('feu_groups','visitors');
        $attr->set_helper(feu_content_attribute_helper::get_instance());
        $tmp[] = $attr;
        return $tmp;
    }

    /**
     * @internal
     * @ignore
     */
    static public function page_type_lang_callback($str)
    {
        $mod = cms_utils::get_module('FrontEndUsers');
        $str = str_replace(' ','_',$str);
        if( $str == 'forgot_password_email' ) $str = 'verify_email';
        $str = 'tpltype_'.$str;
        if( is_object($mod) ) return $mod->Lang($str);
    }

    /**
     * @internal
     * @ignore
     */
    static public function reset_page_type_defaults(CmsLayoutTemplateType $type)
    {
        if( $type->get_originator() != 'FrontEndUsers' ) throw new CmsLogicException('Cannot reset contents for this template type');

        $fn = null;
        switch( $type->get_name() ) {
        case 'login form':
            $fn = 'orig_loginform.tpl';
            break;
        case 'logout form':
            $fn = 'orig_logoutform.tpl';
            break;
        case 'chsettings form':
            $fn = 'orig_changesettings.tpl';
            break;
        case 'forgot password form':
            $fn = 'orig_forgotpassword1.tpl';
            break;
        case 'forgot password email':
            $fn = 'orig_forgotpassword2.tpl';
            break;
        case 'forgot password verify':
            $fn = 'orig_forgotpassword3.tpl';
            break;
        case 'force new password':
            $fn = 'orig_force_newpw_form.tpl';
            break;
        case 'lost username form':
            $fn = 'orig_lostunform_template.tpl';
            break;
        case 'lost username confirm':
            $fn = 'orig_lostunform_confirm_template.tpl';
            break;
        case 'view user':
            $fn = 'orig_viewuser_template.tpl';
            break;
        case 'reset session':
            $fn = 'orig_resetsession_template.tpl';
            break;
        case 'verify_form':
        case 'verify form':
            $fn = 'orig_verifyform.tpl';
            break;
        default:
            throw new \LogicException($type->get_name().' is not a known type for the '.$type->get_originator().' originator');
        }

        $fn = cms_join_path(__DIR__,'templates',$fn);
        if( is_file($fn) ) return @file_get_contents($fn);
    }

    public function set_verification_assistant( verification_assistant $obj )
    {
        if( $this->verification_assistant ) throw new \LogicException('A verification assistant object already exists');
        $this->verification_assistant = $obj;
    }

    protected function get_verification_assistant()
    {
        if( !$this->verification_assistant ) {
            $this->verification_assistant = new email_verify_assistant( $this );
        }
        return $this->verification_assistant;
    }

} // class

// EOF
