<?php
namespace FrontEndUsers;

class email_verify_assistant extends verification_assistant
{
    private $_template;
    private $_subject;
    private $_title;
    private $_msg;
    private $_code_prompt;
    private $_link_prompt;

    private function get_default_template()
    {
        $typename = 'FrontEndUsers::forgot password email';
        $tpl = \CmsLayoutTemplate::load_dflt_by_type($typename);
        if( !$tpl ) {
            audit('','FrontEndUsers','No default template of type '.$typename.' found');
            throw new \RuntimeException('No default template of type '.$typename);
        }
        return $tpl->get_name();
    }

    public function __construct( \FrontEndUsers $feu )
    {
        parent::__construct($feu);

        $this->_template = $this->get_default_template();
        $this->_subject = 'Please verify your account for {sitename}';
        $this->_title = 'Please verify your account';
        $this->_msg = '<p>For security purposes we need to validate your identity.  This is done by verifying that you received this email correctly, and can follow these instructions.</p>
<p>Below, We have provided a unique security code to use to validate your credentials. Please click on the link provided and enter the security code. <em>It may already be filled in on the form</em>.  This will complete the operation and allow you to login to the service.</p>';
        $this->_code_prompt = 'Your security code is:';
        $this->_link_prompt = 'Please click on this link';
    }

    public function SetTemplate($str)
    {
        if( $str ) $this->_template = $str;
    }

    public function SetTitle($str)
    {
        if( $str ) $this->_title = $str;
    }

    public function SetSubject($str)
    {
        if( $str ) $this->_subject = $str;
    }

    public function SetMessage($str)
    {
        if( $str ) $this->_msg = $str;
    }

    public function SetCodePrompt($str)
    {
        if( $str ) $this->_code_prompt = $str;
    }

    public function SetLinkPrompt($str)
    {
        if( $str ) $this->_link_prompt = $str;
    }

    public function send( $uid, $code )
    {
        $uid = (int) $uid;
        $code = trim($code);
        $destpage = $this->get_destpage();
        if( $uid < 1 || empty($code) ) throw new \InvalidArgumentException('Bad data passed to '.__METHOD__);
        $email = $this->feu->GetEmail($uid);
        if( !$email ) throw new \LogicException('Attempt to create a '.__CLASS__.' for a user without a valid email address');

        // code will be sent in the URL
        $url = $this->get_verify_form_url( $uid, $code );

        $tpl = $this->feu->CreateSmartyTemplate($this->_template);
        $tpl->assign('title',$this->_title);
        $tpl->assign('message_forgotpwmail',$this->_msg);
        $tpl->assign('message_forgotpwemail',$this->_msg);
        $tpl->assign('message',$this->_msg);
        $tpl->assign('prompt_code',$this->_code_prompt);
        $tpl->assign('prompt_link',$this->_link_prompt);
        $tpl->assign('destpage',$destpage);
        $tpl->assign('data_code',$code);
        $tpl->assign('subject',$this->_subject);
        $tpl->assign('data_url',$url);
        $body = $tpl->fetch();

        // duplicated code... I know, I know... shaddap
        $smarty = \Smarty_CMS::get_instance();
        $tpl = $smarty->CreateTemplate('string:'.$this->_subject);
        $tpl->assign('title',$this->_title);
        $tpl->assign('message_forgotpwmail',$this->_msg);
        $tpl->assign('message_forgotpwemail',$this->_msg);
        $tpl->assign('message',$this->_msg);
        $tpl->assign('prompt_code',$this->_code_prompt);
        $tpl->assign('prompt_link',$this->_link_prompt);
        $tpl->assign('destpage',$destpage);
        $tpl->assign('data_code',$code);
        $tpl->assign('subject',$this->_subject);
        $tpl->assign('data_url',$url);
        $subject = $tpl->fetch();

        $mailer = new \cms_mailer();
        $mailer->AddAddress($email);
        $mailer->SetBody($body);
        $mailer->SetSubject($subject);
        $mailer->IsHTML(TRUE);
        $mailer->SetPriority(1);
        $mailer->Send();

        audit($uid,$this->feu->GetName(),'Verification email sent');
        $this->feu->add_history($uid,'init_verify');
    }
}